=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-06-01 16:50:59 +0000
+++ bzrlib/builtins.py	2011-06-14 13:41:25 +0000
@@ -6216,6 +6216,7 @@
         ('cmd_resolve', ['resolved'], 'bzrlib.conflicts'),
         ('cmd_conflicts', [], 'bzrlib.conflicts'),
         ('cmd_sign_my_commits', [], 'bzrlib.sign_my_commits'),
+        ('cmd_verify', [], 'bzrlib.sign_my_commits'),
         ('cmd_test_script', [], 'bzrlib.cmd_test_script'),
         ]:
         builtin_command_registry.register_lazy(name, aliases, module_name)

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2009-03-23 14:59:43 +0000
+++ bzrlib/gpg.py	2011-06-14 13:41:25 +0000
@@ -33,6 +33,12 @@
 """)
 
 
+SIGNATURE_VALID = 0
+SIGNATURE_KEY_MISSING = 1
+SIGNATURE_NOT_VALID = 2
+SIGNATURE_NOT_SIGNED = 3
+
+
 class DisabledGPGStrategy(object):
     """A GPG Strategy that makes everything fail."""
 
@@ -111,3 +117,23 @@
                 raise errors.SigningFailed(self._command_line())
             else:
                 raise
+
+    def verify(self, content):
+        import gpgme
+        from StringIO import StringIO
+        context = gpgme.Context()
+        signature = StringIO(content)
+        plain = StringIO()
+        
+        result = context.verify(signature, None, plain)
+        #print "plain: " + plain.getvalue()
+
+        #print "summary: " + str(result[0].summary)
+        if result[0].summary & gpgme.SIGSUM_VALID:
+            return SIGNATURE_VALID
+        if result[0].summary & gpgme.SIGSUM_GREEN:
+            print "green!"
+        if result[0].summary & gpgme.SIGSUM_RED:
+            return SIGNATURE_NOT_VALID
+        if result[0].summary & gpgme.SIGSUM_KEY_MISSING:
+            return SIGNATURE_NOT_VALID

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-05-18 10:24:05 +0000
+++ bzrlib/repository.py	2011-06-14 13:41:25 +0000
@@ -1200,6 +1200,10 @@
         plaintext = testament.as_short_text()
         self.store_revision_signature(gpg_strategy, plaintext, revision_id)
 
+    def verify_revision(self, revision_id, gpg_strategy):
+        signature = self.get_signature_text(revision_id)
+        return gpg_strategy.verify(signature)
+
     def has_signature_for_revision_id(self, revision_id):
         """Query for a revision signature for revision_id in the repository."""
         raise NotImplementedError(self.has_signature_for_revision_id)

=== modified file 'bzrlib/sign_my_commits.py'
--- bzrlib/sign_my_commits.py	2011-04-07 10:36:24 +0000
+++ bzrlib/sign_my_commits.py	2011-06-14 13:41:25 +0000
@@ -26,7 +26,7 @@
 """)
 from bzrlib.commands import Command
 from bzrlib.option import Option
-
+from bzrlib.trace import note
 
 class cmd_sign_my_commits(Command):
     __doc__ = """Sign all commits by a given committer.
@@ -88,3 +88,58 @@
         print 'Signed %d revisions' % (count,)
 
 
+class cmd_verify(Command):
+    __doc__ = """Sign all commits by a given committer.
+
+    If location is not specified the local tree is used.
+    If committer is not specified the default committer is used.
+
+    This does not sign commits that already have signatures.
+    """
+    # Note that this signs everything on the branch's ancestry
+    # (both mainline and merged), but not other revisions that may be in the
+    # repository
+
+    takes_options = [
+            Option('dry-run',
+   help='Don\'t actually sign anything, just print'
+        ' the revisions that would be signed.'),
+            ]
+    takes_args = ['location?', 'committer?']
+
+    def run(self, location=None, committer=None, dry_run=False):
+        if location is None:
+            bzrdir = _mod_bzrdir.BzrDir.open_containing('.')[0]
+        else:
+            # Passed in locations should be exact
+            bzrdir = _mod_bzrdir.BzrDir.open(location)
+        branch = bzrdir.open_branch()
+        repo = branch.repository
+        branch_config = branch.get_config()
+
+        if committer is None:
+            committer = branch_config.username()
+        gpg_strategy = gpg.GPGStrategy(branch_config)
+
+        count = {gpg.SIGNATURE_VALID: 0,
+                 gpg.SIGNATURE_KEY_MISSING: 0,
+                 gpg.SIGNATURE_NOT_VALID: 0,
+                 gpg.SIGNATURE_NOT_SIGNED: 0}
+        result = []
+        for rev_id in repo.get_ancestry(branch.last_revision())[1:]:
+            if not repo.has_signature_for_revision_id(rev_id):
+                result.append([rev_id, gpg.SIGNATURE_NOT_SIGNED])
+                continue
+            rev = repo.get_revision(rev_id)
+            #if rev.committer != committer:
+            #    continue
+            # We have a revision without a signature who has a
+            # matching committer, start signing
+            print rev_id
+            if not dry_run:
+                verification_result = repo.verify_revision(rev_id, gpg_strategy)
+                result.append([rev_id, verification_result])
+                count[verification_result] += 1
+        #print 'Signed %d revisions' % (count,)
+        print "result: " + str(result)
+        print "count: " + str(count)

